@@ -1,10 +1,12 @@
 """A package contaning gaussian-valued models (networks) for all common learnergy modules.
 """
 
-from learnergy.models.gaussian.gaussian_conv_rbm import GaussianConvRBM
+from learnergy.models.gaussian.gaussian_conv_rbm import GaussianConvRBM, GaussianConvRBM4deep
 from learnergy.models.gaussian.gaussian_rbm import (
     GaussianRBM,
+    GaussianRBM4deep,
     GaussianReluRBM,
+    GaussianReluRBM4deep,
     GaussianSeluRBM,
     VarianceGaussianRBM,
 )
