@@ -113,19 +113,14 @@ class GaussianRBM(RBM):
 
         """
 
-        # Calculate samples' activations
         activations = F.linear(samples, self.W.t(), self.b)
 
-        # Creating a Softplus function for numerical stability
+        # Creates a Softplus function for numerical stability
         s = nn.Softplus()
 
-        # Calculate the hidden term
         h = torch.sum(s(activations), dim=1)
-
-        # Calculate the visible term
         v = 0.5 * torch.sum((samples - self.a) ** 2, dim=1)
 
-        # Finally, gathers the system's energy
         energy = v - h
 
         return energy
@@ -144,20 +139,13 @@ class GaussianRBM(RBM):
 
         """
 
-        # Calculating neurons' activations
         activations = F.linear(h, self.W, self.a)
 
-        # If scaling is true
         if scale:
-            # Scale with temperature
             states = torch.div(activations, self.T)
-
-        # If scaling is false
         else:
-            # Gathers the states as usual
             states = activations
 
-        # Passes states through a Sigmoid function
         probs = torch.sigmoid(states)
 
         return probs, states
@@ -180,108 +168,62 @@ class GaussianRBM(RBM):
 
         """
 
-        # Transforming the dataset into training batches
         batches = DataLoader(
             dataset, batch_size=batch_size, shuffle=False, num_workers=0
         )
 
-        # For every epoch
         for epoch in range(epochs):
             logger.info("Epoch %d/%d", epoch + 1, epochs)
 
-            # Calculating the time of the epoch's starting
             start = time.time()
 
-            # Resetting epoch's MSE and pseudo-likelihood to zero
             mse = 0
             pl = 0
 
-            # For every batch
             for samples, _ in tqdm(batches):
                 if self.normalize:
-                    # Normalizing the samples' batch
                     samples = (
                         (samples - torch.mean(samples, 0, True))
                         / (torch.std(samples, 0, True) + 1e-6)
                     ).detach()
 
-                # Flattening the samples' batch
                 samples = samples.reshape(len(samples), self.n_visible)
-
-                # Checking whether GPU is avaliable and if it should be used
                 if self.device == "cuda":
-                    # Applies the GPU usage to the data
                     samples = samples.cuda()
 
                 # Performs the Gibbs sampling procedure
                 _, _, _, _, visible_states = self.gibbs_sampling(samples)
-
-                # Detaching the visible states from GPU for further computation
                 visible_states = visible_states.detach()
 
-                # Calculates the loss for further gradients' computation
                 cost = torch.mean(self.energy(samples)) - torch.mean(
                     self.energy(visible_states)
                 )
 
-                # Initializing the gradient
                 self.optimizer.zero_grad()
-
-                # Computing the gradients
                 cost.backward()
-
-                # Updating the parameters
                 self.optimizer.step()
 
-                # Gathering the size of the batch
                 batch_size = samples.size(0)
 
-                # Calculating current's batch MSE
                 batch_mse = torch.div(
                     torch.sum(torch.pow(samples - visible_states, 2)), batch_size
                 ).detach()
-
-                # Calculating the current's batch logarithm pseudo-likelihood
                 batch_pl = self.pseudo_likelihood(samples).detach()
 
-                # Summing up to epochs' MSE and pseudo-likelihood
                 mse += batch_mse
                 pl += batch_pl
 
-            # Normalizing the MSE and pseudo-likelihood with the number of batches
             mse /= len(batches)
             pl /= len(batches)
 
-            # Calculating the time of the epoch's ending
             end = time.time()
 
-            # Dumps the desired variables to the model's history
             self.dump(mse=mse.item(), pl=pl.item(), time=end - start)
 
             logger.info("MSE: %f | log-PL: %f", mse, pl)
 
         return mse, pl
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        """Performs a forward pass over the data.
-
-        Args:
-            x: An input tensor for computing the forward pass.
-
-        Returns:
-            (torch.Tensor): A tensor containing the RBM's outputs.
-
-        """
-
-        if self.input_normalize:
-            # Normalizing the samples
-            x = ((x - torch.mean(x, 0, True)) / (torch.std(x, 0, True) + 1e-6)).detach()
-
-        # Calculates the outputs of the model
-        x, _ = self.hidden_sampling(x)
-
-        return x
-
     def reconstruct(
         self, dataset: torch.utils.data.Dataset
     ) -> Tuple[float, torch.Tensor]:
@@ -297,55 +239,56 @@ class GaussianRBM(RBM):
 
         logger.info("Reconstructing new samples ...")
 
-        # Resetting MSE to zero
         mse = 0
-
-        # Defining the batch size as the amount of samples in the dataset
         batch_size = len(dataset)
 
-        # Transforming the dataset into training batches
         batches = DataLoader(
             dataset, batch_size=batch_size, shuffle=False, num_workers=0
         )
 
-        # For every batch
         for samples, _ in tqdm(batches):
             if self.normalize:
-                # Normalizing the samples' batch
                 samples = (
                     (samples - torch.mean(samples, 0, True))
                     / (torch.std(samples, 0, True) + 1e-6)
                 ).detach()
 
-            # Flattening the samples' batch
             samples = samples.reshape(len(samples), self.n_visible)
-
-            # Checking whether GPU is avaliable and if it should be used
             if self.device == "cuda":
-                # Applies the GPU usage to the data
                 samples = samples.cuda()
 
-            # Calculating positive phase hidden probabilities and states
             _, pos_hidden_states = self.hidden_sampling(samples)
-
-            # Calculating visible probabilities and states
             visible_probs, visible_states = self.visible_sampling(pos_hidden_states)
 
-            # Calculating current's batch reconstruction MSE
             batch_mse = torch.div(
                 torch.sum(torch.pow(samples - visible_states, 2)), batch_size
             )
-
-            # Summing up the reconstruction's MSE
             mse += batch_mse
 
-        # Normalizing the MSE with the number of batches
         mse /= len(batches)
 
         logger.info("MSE: %f", mse)
 
         return mse, visible_probs
 
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """Performs a forward pass over the data.
+
+        Args:
+            x: An input tensor for computing the forward pass.
+
+        Returns:
+            (torch.Tensor): A tensor containing the RBM's outputs.
+
+        """
+
+        if self.input_normalize:
+            x = ((x - torch.mean(x, 0, True)) / (torch.std(x, 0, True) + 1e-6)).detach()
+
+        x, _ = self.hidden_sampling(x)
+
+        return x
+
 
 class GaussianReluRBM(GaussianRBM):
     """A GaussianReluRBM class provides the basic implementation for
@@ -421,17 +364,11 @@ class GaussianReluRBM(GaussianRBM):
 
         """
 
-        # Calculating neurons' activations
         activations = F.linear(v, self.W.t(), self.b)
 
-        # If scaling is true
         if scale:
-            # Calculate probabilities with temperature
             probs = F.relu(torch.div(activations, self.T))
-
-        # If scaling is false
         else:
-            # Calculate probabilities as usual
             probs = F.relu(activations)
 
         # Current states equals probabilities
@@ -516,17 +453,11 @@ class GaussianSeluRBM(GaussianRBM):
 
         """
 
-        # Calculating neurons' activations
         activations = F.linear(v, self.W.t(), self.b)
 
-        # If scaling is true
         if scale:
-            # Calculate probabilities with temperature
             probs = F.selu(torch.div(activations, self.T))
-
-        # If scaling is false
         else:
-            # Calculate probabilities as usual
             probs = F.selu(activations)
 
         # Current states equals probabilities
@@ -591,15 +522,10 @@ class VarianceGaussianRBM(RBM):
             use_gpu,
         )
 
-        # Variance parameter
         self.sigma = nn.Parameter(torch.ones(n_visible))
-
-        # Updating optimizer's parameters with `sigma`
         self.optimizer.add_param_group({"params": self.sigma})
 
-        # Re-checks if current device is CUDA-based due to new parameter
         if self.device == "cuda":
-            # If yes, re-uses CUDA in the whole class
             self.cuda()
 
         logger.info("Class overrided.")
@@ -628,22 +554,15 @@ class VarianceGaussianRBM(RBM):
 
         """
 
-        # Calculating neurons' activations
         activations = F.linear(
             torch.div(v, torch.pow(self.sigma, 2)), self.W.t(), self.b
         )
 
-        # If scaling is true
         if scale:
-            # Calculate probabilities with temperature
             probs = torch.sigmoid(torch.div(activations, self.T))
-
-        # If scaling is false
         else:
-            # Calculate probabilities as usual
             probs = torch.sigmoid(activations)
 
-        # Sampling current states
         states = torch.bernoulli(probs)
 
         return probs, states
@@ -662,20 +581,15 @@ class VarianceGaussianRBM(RBM):
 
         """
 
-        # Calculating neurons' activations
         activations = F.linear(h, self.W, self.a)
 
-        # Checks if device is CPU-based
         if self.device == "cpu":
-            # If yes, variance needs to have size equal to (batch_size, n_visible)
+            # If on cpu, variance needs to have size equal to (batch_size, n_visible)
             sigma = torch.repeat_interleave(self.sigma, activations.size(0), dim=0)
-
-        # If it is GPU-based
         else:
             # Variance needs to have size equal to (n_visible)
             sigma = self.sigma
 
-        # Sampling current states from a Gaussian distribution
         states = torch.normal(activations, torch.pow(sigma, 2))
 
         return states, activations
@@ -691,23 +605,236 @@ class VarianceGaussianRBM(RBM):
 
         """
 
-        # Calculating the potency of variance
         sigma = torch.pow(self.sigma, 2)
-
-        # Calculate samples' activations
         activations = F.linear(torch.div(samples, sigma), self.W.t(), self.b)
 
-        # Creating a Softplus function for numerical stability
+        # Createa a Softplus function for numerical stability
         s = nn.Softplus()
 
-        # Calculate the hidden term
         h = torch.sum(s(activations), dim=1)
-
-        # Calculate the visible term
-        # Note that this might be improved
         v = torch.sum(torch.div(torch.pow(samples - self.a, 2), 2 * sigma), dim=1)
 
-        # Finally, gathers the system's energy
         energy = -v - h
 
         return energy
+
+class GaussianRBM4deep(GaussianRBM):
+    """A GaussianRBM class provides the basic implementation for
+    Gaussian-Bernoulli Restricted Boltzmann Machines (with standardization).
+
+    Note that this classes normalize the data
+    as it uses variance equals to one throughout its learning procedure.
+
+    This is a trick to ease the calculations of the hidden and
+    visible layer samplings, as well as the cost function.
+
+    References:
+        K. Cho, A. Ilin, T. Raiko.
+        Improved learning of Gaussian-Bernoulli restricted Boltzmann machines.
+        International conference on artificial neural networks (2011).
+
+    """
+
+    def __init__(
+        self,
+        n_visible: Optional[int] = 128,
+        n_hidden: Optional[int] = 128,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.1,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        temperature: Optional[float] = 1.0,
+        use_gpu: Optional[bool] = False,
+        normalize: Optional[bool] = True,
+        input_normalize: Optional[bool] = True,
+    ) -> None:
+        """Initialization method.
+
+        Args:
+            n_visible: Amount of visible units.
+            n_hidden: Amount of hidden units.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            temperature: Temperature factor.
+            use_gpu: Whether GPU should be used or not.
+            normalize: Whether or not to use batch normalization.
+            input_normalize: Whether or not to normalize inputs.
+
+        """
+
+        self._normalize = normalize
+        self._input_normalize = input_normalize
+
+        logger.info("Overriding class: RBM -> GaussianRBM.")
+
+        super(GaussianRBM4deep, self).__init__(
+            n_visible,
+            n_hidden,
+            steps,
+            learning_rate,
+            momentum,
+            decay,
+            temperature,
+            use_gpu,
+        )
+
+        logger.info("Class overrided.")
+
+  
+    def fit(
+        self,
+        dataset: torch.utils.data.Dataset,
+        batch_size: Optional[int] = 128,
+        epochs: Optional[int] = 1,
+    ) -> Tuple[float, float]:
+        """Fits a new GaussianRBM model.
+
+        Args:
+            dataset: A Dataset object containing the training data.
+            batch_size: Amount of samples per batch.
+            epochs: Number of training epochs.
+
+        Returns:
+            (Tuple[float, float]): MSE (mean squared error) and log pseudo-likelihood from the training step.
+
+        """
+
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=True, num_workers=0
+        )
+
+        for epoch in range(epochs):
+            start = time.time()
+            mse = 0
+            pl = 0
+
+            for _, (samples, _) in enumerate(batches):
+                if self.normalize:
+                    samples = (
+                        (samples - torch.mean(samples, 0, True))
+                        / (torch.std(samples, 0, True) + 1e-6)
+                    ).detach()
+
+                samples = samples.reshape(len(samples), self.n_visible)
+                if self.device == "cuda":
+                    samples = samples.cuda()
+
+                # Performs the Gibbs sampling procedure
+                _, _, _, _, visible_states = self.gibbs_sampling(samples)
+                visible_states = visible_states.detach()
+
+                cost = torch.mean(self.energy(samples)) - torch.mean(
+                    self.energy(visible_states)
+                )
+
+                self.optimizer.zero_grad()
+                cost.backward()
+                self.optimizer.step()
+
+                batch_size = samples.size(0)
+
+                batch_mse = torch.div(
+                    torch.sum(torch.pow(samples - visible_states, 2)), batch_size
+                ).detach()
+                batch_pl = self.pseudo_likelihood(samples).detach()
+
+                mse += batch_mse
+                pl += batch_pl
+
+            mse /= len(batches)
+            pl /= len(batches)
+
+            end = time.time()
+
+            self.dump(mse=mse.item(), pl=pl.item(), time=end - start)
+
+        return mse, pl
+
+
+class GaussianReluRBM4deep(GaussianRBM4deep):
+    """A GaussianReluRBM class provides the basic implementation for
+    Gaussian-ReLU Restricted Boltzmann Machines (for raw pixels values).
+
+    Note that this class requires raw data (integer-valued)
+    in order to model the image covariance into a latent ReLU layer.
+
+    References:
+        G. Hinton. A practical guide to training restricted Boltzmann machines.
+        Neural networks: Tricks of the trade (2012).
+
+    """
+
+    def __init__(
+        self,
+        n_visible: Optional[int] = 128,
+        n_hidden: Optional[int] = 128,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.001,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        temperature: Optional[float] = 1.0,
+        use_gpu: Optional[bool] = False,
+        normalize: Optional[bool] = True,
+        input_normalize: Optional[bool] = True,
+    ) -> None:
+        """Initialization method.
+
+        Args:
+            n_visible: Amount of visible units.
+            n_hidden: Amount of hidden units.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            temperature: Temperature factor.
+            use_gpu: Whether GPU should be used or not.
+            normalize: Whether or not to use batch normalization.
+            input_normalize: Whether or not to normalize inputs.
+
+        """
+
+        logger.info("Overriding class: GaussianRBM -> GaussianReluRBM.")
+
+        # Override its parent class
+        super(GaussianReluRBM4deep, self).__init__(
+            n_visible,
+            n_hidden,
+            steps,
+            learning_rate,
+            momentum,
+            decay,
+            temperature,
+            use_gpu,
+            normalize,
+            input_normalize,
+        )
+
+        logger.info("Class overrided.")
+
+    def hidden_sampling(
+        self, v: torch.Tensor, scale: Optional[bool] = False
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
+        """Performs the hidden layer sampling, i.e., P(h|v).
+
+        Args:
+            v: A tensor incoming from the visible layer.
+            scale: A boolean to decide whether temperature should be used or not.
+
+        Returns:
+            (Tuple[torch.Tensor, torch.Tensor]): The probabilities and states of the hidden layer sampling.
+
+        """
+
+        activations = F.linear(v, self.W.t(), self.b)
+
+        if scale:
+            probs = F.relu(torch.div(activations, self.T))
+        else:
+            probs = F.relu(activations)
+
+        # Current states equals probabilities
+        states = probs
+
+        return probs, states
\ No newline at end of file
